<%-- $Id: view_bioplate.jsp 7604 2019-02-25 12:19:50Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Martin
	@version 2.10
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Group"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.AnnotationSet"
	import="net.sf.basedb.core.BioPlate"
	import="net.sf.basedb.core.BioWell"
	import="net.sf.basedb.core.MeasuredBioMaterial"
	import="net.sf.basedb.core.PlateGeometry"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.MultiPermissions"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.core.Project"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.ChangeHistoryUtil"
	import="net.sf.basedb.clients.web.PermissionUtil"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.util.formatter.WellCoordinateFormatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction" 
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.Collections"
	import="java.util.Date"
	import="java.util.Map"
	import="java.util.Set"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="p" uri="/WEB-INF/path.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.BIOPLATE;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.ITEM);
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String tab = Values.getStringOrNull(request.getParameter("tab"));
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);

	String title = null;
	BioPlate bioplate = BioPlate.getById(dc, itemId);
	Item bioMaterialType = bioplate.getBioPlateType().getBioMaterialType();
	PlateGeometry geometry = bioplate.getPlateGeometry();
	
	final boolean writePermission = bioplate.hasPermission(Permission.WRITE);
	final boolean deletePermission = bioplate.hasPermission(Permission.DELETE);
	final boolean sharePermission = bioplate.hasPermission(Permission.SET_PERMISSION);
	final boolean setOwnerPermission = bioplate.hasPermission(Permission.SET_OWNER);
	final boolean isRemoved = bioplate.isRemoved();
	final boolean isUsed = isRemoved && bioplate.isUsed();
	final boolean deletePermanentlyPermission = deletePermission && !isUsed;
	final boolean isOwner = bioplate.isOwner();
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, guiContext, bioplate);
	ExtensionsInvoker<ButtonAction> invoker = ToolbarUtil.useExtensions(jspContext);
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	%>
	<base:page title="<%=title%>" id="view-page">
	<base:head scripts="tabcontrol-2.js,table.js,~bioplates.js" 
		styles="toolbar.css,table.css,headertabcontrol.css,path.css,plate.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<p:path><p:pathelement 
			title="Bioplates" href="<%="index.jsp?ID="+ID%>" /><p:pathelement 
			title="<%=HTML.encodeTags(bioplate.getName())%>" /></p:path>
		<div id="page-data" data-item-id="<%=itemId%>"></div>
		
		<t:tabcontrol id="main" 
			subclass="content mastertabcontrol"
			active="<%=tab%>">
		<t:tab id="properties" title="Properties">
			<div>
			<table class="fullform bottomborder">
			<tr>
				<th class="itemstatus">
					<base:icon 
						image="shared.png" 
						visible="<%=bioplate.isShared()%>"
						tooltip="This item is shared to other users, groups and/or projects"
					/>
					<base:icon 
						id="btnDeletePermanently"
						image="deleted.png"
						tooltip="This item has been flagged for deletion. Click to delete it now."
						enabled="<%=deletePermanentlyPermission %>"
						visible="<%=isRemoved%>" 
					/>
					<base:icon
						id="btnUsingItems"
						image="used.png" 
						tooltip="This item is used by other items and can't be permanently deleted. Show the items that are using this one"
						visible="<%=isRemoved && isUsed%>" />
				</th>
				<td style="padding: 0px;" colspan="2">
					<tbl:toolbar subclass="bottomborder bg-filled-50">
						<tbl:button 
							id="btnEdit"
							disabled="<%=!writePermission%>" 
							image="edit.png" 
							title="Edit&hellip;" 
							tooltip="<%=writePermission ? "Edit this bioplate" : "You do not have permission to edit this bioplate"%>" 
						/>
						<tbl:button 
							id="btnDelete"
							disabled="<%=!deletePermission%>" 
							image="delete.png" 
							title="Delete"
							visible="<%=!bioplate.isRemoved()%>"
							tooltip="<%=deletePermission ? "Delete this bioplate" : "You do not have permission to delete this bioplate"%>" 
						/>
						<tbl:button 
							id="btnRestore"
							disabled="<%=!writePermission%>" 
							image="restore.png" 
							title="Restore"
							visible="<%=bioplate.isRemoved()%>"
							tooltip="<%=writePermission ? "Restore this bioplate" : "You do not have permission to restore this bioplate"%>" 
						/>
						<tbl:button 
							id="btnShare"
							disabled="<%=!sharePermission%>"
							image="share.png"
							title="Share&hellip;" 
							tooltip="<%=sharePermission ? "Share this bioplate to other user, groups and projects" : "You do not have permission to share this bioplate"%>"
						/>
						<tbl:button 
							id="btnSetOwner"
							disabled="<%=!setOwnerPermission%>"
							image="take_ownership.png"
							title="Set owner&hellip;"
							tooltip="<%=setOwnerPermission ? "Change owner of this item" : "You do not have permission to change ownership of this item"%>"
						/>
						<tbl:button 
							id="btnMoveBioMaterial"
							image="move_to_plate.png" 
							title="Move biomaterial&hellip;" 
							tooltip="Move biomaterial on this plate to another plate" 
						/>
						<tbl:button 
							id="btnCreateChildBioPlate"
							image="add.png" 
							title="<%="Create child bioplate" + (bioMaterialType == Item.EXTRACT ? "/bioassay" : "") +"&hellip;"%>" 
							tooltip="Create one or more child biomaterial plates" 
							visible="<%=bioMaterialType != null%>"
						/>
						<tbl:button 
							id="btnImport"
							image="import.png" 
							data-plugin-type="IMPORT"  
							title="Import&hellip;" 
							tooltip="Import data" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
						/>
						<tbl:button 
							id="btnExport"
							image="export.png"
							data-plugin-type="EXPORT" 
							title="Export&hellip;" 
							tooltip="Export data" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
						/>
						<tbl:button 
							id="btnRunPlugin"
							image="runplugin.png"  
							data-plugin-type="OTHER" 
							title="Run plugin&hellip;" 
							tooltip="Run a plugin" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
						/>
						<ext:render extensions="<%=invoker%>" context="<%=jspContext%>" 
							wrapper="<%=new PrefixSuffixRenderer<ButtonAction>(jspContext, "<td>", "</td>") %>"/>
						<tbl:button
							image="help.png"
							subclass="auto-init"
							data-auto-init="help"
							data-help-id="plate.view.properties"
							title="Help&hellip;"
							tooltip="Get help about this page"
						/>
					</tbl:toolbar>
				</td>
			</tr>
			<tr>
				<th>Name</th>
				<td><%=HTML.encodeTags(bioplate.getName())%></td>
				<td rowspan="12" style="width: 50%; vertical-align: top;">
				
				<table class="plate" style="margin-top: 1em;" id="bioplate">
				<%
				int columns = geometry.getColumns();
				int rows = geometry.getRows();
				WellCoordinateFormatter rowF = new WellCoordinateFormatter(true);
				WellCoordinateFormatter colF = new WellCoordinateFormatter(false);
				%>
				<tr>
					<td class="bg-filled-100">
						<div class="notify" id="wellInfo" style="display:none; width: 20em; min-height: 3em;"></div>
					</td>
				<%
				for (int c = 0; c < columns; ++c)
				{
					%>
					<td class="columnheader bg-filled-100"><%=colF.format(c)%></td>
					<%
				}
				%>
				</tr>
				<%
				for (int r = 0; r < rows; ++r)
				{
					String row = rowF.format(r);
					%>
					<tr><td class="rowheader bg-filled-100"><%=row%></td>
					<%
					for (int c = 0; c < columns; ++c)
					{
						BioWell well = bioplate.getBioWell(r, c);
						if (well == null) continue;
						boolean isEmpty = well.isEmpty();
						boolean hasBeenUsed = well.hasBeenUsed();
						boolean canAddBioMaterial = well.canAddBioMaterial();
						boolean canClearBioMaterial = well.canClearBioMaterial();
						boolean usePermission = well.hasPermission(Permission.USE);
						
						boolean deniedBioMaterial = false;
						boolean allowEdit = false;
						boolean showInfo = false;
						
						String cls = "well auto-init";
						String tooltip = "";
						if (isEmpty)
						{
							cls += " empty";
							if (usePermission && canAddBioMaterial)
							{
								// The used is allowed to add biomaterial to the empty well
								cls += " editable";
								allowEdit = true;
								tooltip = "Add biomaterial to this well";
							}
							else if (hasBeenUsed) 
							{
								// The well has been used and it is not allowed to add biomaterial
								cls += " wasused";
								showInfo = true;
								tooltip = "This well has already been used";
							}
							else
							{
								// The user doesn't have permission to add biomaterial
								cls += " locked";
								tooltip = "You are not allowed to add biomaterial to this well";
							}
						}
						else
						{
							cls += " used";
							try
							{
								MeasuredBioMaterial bioMaterial = well.getBioMaterial();
								showInfo = true;
								if (usePermission && canClearBioMaterial && bioMaterial.hasPermission(Permission.WRITE))
								{
									// The user is allowed to replace/remove the biomaterial in the well
									cls += " editable";
									allowEdit = true;
									if (canAddBioMaterial)
									{
										// Replace and/or remove
										tooltip = "Change biomaterial in this well";
									}
									else
									{
										// Remove only
										tooltip = "Remove the biomaterial from this well";
									}
								}
								else
								{
									cls += " locked";
									tooltip = "This well is locked for modifications";
								}
							}
							catch (PermissionDeniedException ex)
							{
								cls += " denied";
								tooltip = "Access denied";
							}
						}
						%>
						<td class="<%=cls%>" 
							id="well.<%=well.getId()%>"
							data-auto-init="well"
							data-allow-edit="<%=allowEdit ? 1 : 0 %>"
							data-show-info="<%=showInfo ? 1 : 0 %>"
							data-item-id="<%=well.getId()%>"
							data-coordinate="<%=well.getCoordinate()%>"
							title="<%=tooltip%>"></td>
						<%
					}
					%>
					</tr>
					<%
				}
				%>
				</table>
				
				
				</td>
			</tr>
			<tr>
				<th>Barcode</th>
				<td><%=HTML.encodeTags(bioplate.getBarcode())%></td>
			</tr>
			<tr>
				<th>Created</th>
				<td><%=dateFormatter.format(bioplate.getEventDate())%></td>
			</tr>
			<tr>
				<th>Registered</th>
				<td><%=dateFormatter.format(bioplate.getEntryDate())%></td>
			</tr>
			<tr>
				<th>Destroyed</th>
				<td><%=bioplate.isDestroyed() ? "yes" : "no"%></td>
			</tr>
			<tr>
				<th>Bioplate type</th>
				<td><base:propertyvalue item="<%=bioplate%>" property="bioPlateType" /></td>
			</tr>
			<tr>
				<th>Geometry</th>
				<td><base:propertyvalue item="<%=bioplate%>" property="plateGeometry" /></td>
			</tr>
			<tr>
				<th>Free (total) wells</th>
				<td><%=bioplate.getFreeWells()%> (<%=bioplate.getTotalWells()%>)</td>
			</tr>
			<tr>
				<th>Storage location</th>
				<td><base:propertyvalue item="<%=bioplate%>" property="freezer" /></td>
			</tr>
			<tr>
				<th class="subprompt"></th>
				<td>
					<b>Section</b>: <%=HTML.encodeTags(Values.getString(bioplate.getSection(), "-")) %>;
					<b>Tray</b>: <%=HTML.encodeTags(Values.getString(bioplate.getTray(), "-")) %>;
					<b>Position</b>: <%=HTML.encodeTags(Values.getString(bioplate.getPosition(), "-")) %>
				</td>
			</tr>
			<tr>
				<th>Owner</th>
				<td><base:propertyvalue item="<%=bioplate%>" property="owner" /></td>
			</tr>
			<tr>
				<th>Permissions</th>
				<td><%=PermissionUtil.getFullPermissionNames(bioplate)%></td>
			</tr>
			<tr class="dynamic">
				<th>Description</th>
				<td><%=HTML.niceFormat(bioplate.getDescription())%></td>
			</tr>
		</table>
		</div>
			
				<jsp:include page="../../common/anytoany/list_anytoany.jsp">
					<jsp:param name="ID" value="<%=ID%>" />
					<jsp:param name="item_type" value="<%=itemType.name()%>" />
					<jsp:param name="item_id" value="<%=itemId%>" />
					<jsp:param name="title" value="Other items related to this bioplate" />
				</jsp:include>
				<jsp:include page="../../common/share/list_share.jsp">
					<jsp:param name="ID" value="<%=ID%>" />
					<jsp:param name="item_type" value="<%=itemType.name()%>" />
					<jsp:param name="item_id" value="<%=itemId%>" />
					<jsp:param name="title" value="Shared to" />
				</jsp:include>
			</t:tab>
			
			<t:tab id="annotations" title="Annotations" 
				tooltip="View annotation values" clazz="white">
				<jsp:include page="../../common/annotations/list_frameset.jsp">
					<jsp:param name="item_type" value="<%=itemType.name()%>" />
					<jsp:param name="item_id" value="<%=itemId%>" />
					<jsp:param name="ID" value="<%=ID%>" />
				</jsp:include>
			</t:tab>

			<t:tab id="wells" title="Wells" />
			<t:tab id="events" title="Plate events" />
			<t:tab id="overview" title="Overview" 
				tooltip="Display a tree overview of related items">
				<jsp:include page="../../common/overview/overview.jsp">
					<jsp:param name="item_type" value="<%=itemType.name()%>" />
					<jsp:param name="item_id" value="<%=itemId%>" />
					<jsp:param name="ID" value="<%=ID%>" />
				</jsp:include>
			</t:tab>			
			<t:tab id="history" title="History"
				tooltip="Displays a log with the history of this item"
				visible="<%=ChangeHistoryUtil.showChangeHistoryTab(sc)%>">
				<jsp:include page="../../common/history/frameset.jsp">
					<jsp:param name="item_type" value="<%=itemType.name()%>" />
					<jsp:param name="item_id" value="<%=itemId%>" />
					<jsp:param name="ID" value="<%=ID%>" />
				</jsp:include>
			</t:tab>
		</t:tabcontrol>
		
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
