<%-- $Id: inherit.jsp 7605 2019-02-26 10:10:15Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	This page is used to display and modify the attributes of
	the of a news item

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.Type"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Annotatable"
	import="net.sf.basedb.core.AnnotationSet"
	import="net.sf.basedb.core.Annotation"
	import="net.sf.basedb.core.RawBioAssay"
	import="net.sf.basedb.core.PhysicalBioAssay"
	import="net.sf.basedb.core.DerivedBioAssay"
	import="net.sf.basedb.core.Extract"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.util.AnnotationUtil"
	import="net.sf.basedb.core.Subtypable"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.util.Tree"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="java.util.List"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.Map"
	import="java.util.HashMap"
	import="org.json.simple.JSONArray"
	import="org.json.simple.JSONObject"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%!
private void loadParents(DbControl dc, Set<AnnotationSet> parentAnnotations, Set<Annotatable> parentItems, Annotatable current)
{
	Set<Annotatable> parents = AnnotationUtil.getAllAnnotatableParentItems(dc, current, null);
	for (Annotatable parent : parents)
	{
		if (parent != null && parentItems.add(parent))
		{
			if (parent.hasPermission(Permission.USE) && parent.isAnnotated())
			{
				parentAnnotations.add(parent.getAnnotationSet());
			}
		}
	}
}

JSONArray generateJoustTree(Set<AnnotationSet> parentAnnotations, Set<AnnotationSet> nonParents, Set<Annotation> inheritedAnnotations)
{
	String itemIcon = "Annotatable";
	String annotationIcon = "Annotation";
	String parameterIcon = "Parameter";
	String nonParentIcon = "NonParent";
	JSONArray json = new JSONArray();
	
	for (AnnotationSet as : parentAnnotations)
	{
		Annotatable item = null;
		String itemType = as.getItemType().toString();
		try
		{
			item = as.getItem();
		}
		catch (PermissionDeniedException ex)
		{}
		if (item instanceof Subtypable)
		{
			try
			{
				ItemSubtype subtype = ((Subtypable)item).getItemSubtype();
				if (subtype != null) itemType = subtype.getName();
			}
			catch (PermissionDeniedException ex)
			{}
		}
		
		String icon = nonParents.contains(as) ? nonParentIcon : itemIcon;
		
		String joustId = "as-"+as.getId();
		String input = "<input type=\"checkbox\""+
			" name=\""+joustId+"\">";
		String name = item == null ? "<i>- denied -</i>" : 
			HTML.encodeTags(((Nameable)item).getName()) + " <span class=\"itemsubtype\">(" + itemType + ")</span>";
		
		ItemQuery<Annotation> query = as.getAnnotations(Annotation.Source.PRIMARY);
		query.restrict(Restrictions.eq(Hql.property("annotationType.disableInheritance"), Expressions.bool(false)));
		List<Annotation> annotations = query.list(as.getDbControl());
		if (annotations.size() == 0) continue; // With the next AnnotationSet/item
			
		JSONObject jsonJoust = null;

		SessionControl sc = as.getSessionControl();
		for (Annotation a : annotations)
		{
			Formatter<Object> formatter = FormatterFactory.getAnnotationFormatter(sc, a, null);
			boolean inherited = inheritedAnnotations != null && inheritedAnnotations.contains(a);
			if (inherited) continue;
			
			if (jsonJoust == null)
			{
				jsonJoust = newJoustEntry(null, icon, input+name, joustId);
				jsonJoust.put("isOpen", 1);
				jsonJoust.put("type", "annotation-set");
				jsonJoust.put("externalId", as.getId());
				json.add(jsonJoust);
			}
			
			AnnotationType at = a.getAnnotationType();
			String annotationName = at.getName();
			boolean isParameter = at.isProtocolParameter();

			joustId = "a-"+a.getId();
			String values = Values.getString(a.getValues(null), ", ", true, formatter);
			name = HTML.encodeTags(annotationName +
				" [" + Values.trimString(values, 20)+"]");
			input = "<input type=\"checkbox\""+
				" name=\""+joustId+"\""+
				(inherited ? " checked" : "")+
				">";
			
			JSONObject jsonAnnotation = newJoustEntry(jsonJoust, isParameter ? parameterIcon : annotationIcon, input+name, joustId);
			jsonAnnotation.put("type", "annotation");
			jsonAnnotation.put("externalId", a.getId());
			jsonAnnotation.put("annotationType", HTML.encodeTags(annotationName));
			jsonAnnotation.put("values", HTML.encodeTags(values));
			jsonAnnotation.put("inherited", inherited ? 1 : 0);
			jsonAnnotation.put("modified", 0);
		}
	}
	return json;
}

JSONObject newJoustEntry(JSONObject jsonParent, String icon, String text, String id)
{
	JSONObject jsonJoust = new JSONObject();
	jsonJoust.put("icon", icon);
	jsonJoust.put("text", text);
	jsonJoust.put("id", id);
	if (jsonParent != null)
	{
		JSONArray jsonChildren = (JSONArray)jsonParent.get("children");
		if (jsonChildren == null)
		{
			jsonChildren = new JSONArray();
			jsonParent.put("children", jsonChildren);
		}
		jsonChildren.add(jsonJoust);
	}
	return jsonJoust;
}
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final Item itemType = Item.valueOf(request.getParameter("item_type"));
final int itemId = Values.getInt(request.getParameter("item_id"));

//final boolean standalone = Values.getBoolean(request.getParameter("standalone"));

final DbControl dc = sc.newDbControl();
ItemResultList<AnnotationType> annotationTypes = null;
try
{
	final Annotatable item = itemId == 0 ? null : (Annotatable)itemType.getById(dc, itemId);
	Set<AnnotationSet> parentAnnotations = new java.util.LinkedHashSet<AnnotationSet>();
	Set<Annotatable> parentItems = new HashSet<Annotatable>();
	Set<AnnotationSet> nonParents = new HashSet<AnnotationSet>();

	if (item != null)
	{
		// Get all annotated parents and their annotation sets
		loadParents(dc, parentAnnotations, parentItems, item);
	}

	String title = "Inherit annotations to " + 
		HTML.encodeTags((item instanceof Nameable ? ((Nameable)item).getName() : 
		(item == null ? " new item" : item.toString())));

	// Parent items may have been submitted by caller
	String[] parents = request.getParameterValues("parents");
	// Each parameter is ITEMTYPE:ID:ID:...
	if (parents != null)
	{
		for (String parent : parents)
		{
			String[] temp = parent.split(":");
			Item parentType = Item.valueOf(temp[0]);
			for (int i = 1; i < temp.length; ++i)
			{
				int parentId = Values.getInt(temp[i], -1);
				if (parentId != -1) 
				{
					Annotatable parentItem = (Annotatable)parentType.getById(dc, parentId);
					if (parentItems.add(parentItem))
					{
						// This is a new parent
						if (parentItem.hasPermission(Permission.USE) && parentItem.isAnnotated()) 
						{
							parentAnnotations.add(parentItem.getAnnotationSet());
						}
						loadParents(dc, parentAnnotations, parentItems, parentItem);
					}
				}
			}
		}
	}
	
	// Get the currently inherited annotations
	final AnnotationSet as = item != null && item.isAnnotated() ? item.getAnnotationSet() : null;
	Set<Annotation> inheritedAnnotations = null;
	
	if (as != null)
	{
		ItemQuery<Annotation> inheritedQuery = as.getAnnotations(null);
		inheritedQuery.restrict(
				Restrictions.neq(
					Hql.property("source"), 
					Expressions.integer(Annotation.Source.PRIMARY.ordinal())
			));
		
		// Ignore annotations marked for DELETE (so it is possible to re-select them)
		String d = request.getParameter("deleted");
		if (d != null)
		{
			Integer[] deleted = Values.getInt(d.split(","));
			if (deleted.length > 0)
			{
				inheritedQuery.restrict(Restrictions.not(Restrictions.in(Hql.property("id"), Expressions.parameter("deleted"))));
				inheritedQuery.setParameter("deleted", java.util.Arrays.asList(deleted), Type.INT);
			}
		}
				
		inheritedQuery.order(Orders.asc(Hql.property("annotationSet")));
		inheritedQuery.order(Orders.asc(Hql.property("annotationType.name")));
		inheritedAnnotations = new HashSet<Annotation>();
		for (Annotation a : inheritedQuery.list(dc))
		{
			a = a.getInheritedFrom();
			if (a != null)
			{
				inheritedAnnotations.add(a);
				AnnotationSet from = a.getAnnotationSet();
				if (!parentAnnotations.contains(from) && from.hasPermission(Permission.USE))
				{
					parentAnnotations.add(from);
					nonParents.add(from);
				}
			}
		}
	}
	JSONArray json = generateJoustTree(parentAnnotations, nonParents, inheritedAnnotations);
	%>
	<base:page type="popup" title="Inherit annotations">
	<base:head  scripts="joust-2.js,~inherit.js" styles="joust-2.css">
	<style>
	.selected .itemsubtype 
	{
		color: #E8E8E8;
	}
	</style>
	</base:head>
	<base:body>
		
	<h1><%=title%> <base:help helpid="annotations.inherit" /></h1>
	<div class="content bottomborder">
	
	<div id="page-data" class="datacontainer"
		data-callback="<%=HTML.encodeTags(request.getParameter("callback"))%>"
	></div>
	
	<form name="annotations">	
	<div class="absolutefull">
	
		<div class="absolutefull bg-filled-100 rightborder" style="width: 28em;">
		
			<div class="absolutefull bottomborder" style="height: 2em;">
			<%
			if (parentAnnotations.size() > 0) 
			{
				%>
				<table style="width: 100%;">
				<tr>
					<td style="padding-left: 4px; padding-right: 4px;"><base:icon image="filter.png"/></td>
					<td style="width: 100%; padding-top: 1px;">
						<input type="text" name="quickFilter" id="quickFilter" 
							class="auto-init" data-auto-init="focus"
							style="width: calc(100% - 10px);"
							title="Type to filter annotation types">
					</td>
				</table>
				<%
			}
			%>
			</div>
			<div class="absolutefull parameterlist" style="top: 2em; bottom: 3em;">
				<div id="joust" class="joust absolutefull"
					data-joust-tree="<%=HTML.encodeTags(json.toJSONString()) %>"
					>
					<%
					if (parentAnnotations.size() == 0) 
					{
						%>
						<div class="messagecontainer error" style="margin: 2em;">There are no parent items with annotations.</div>
						<%
					}
					%>
					<div id="noMatchingFilter" class="messagecontainer error" style="margin: 2em; display: none;">No annotations matches the filter.</div>
				</div>
			</div>
			
			<div class="absolutefull topborder" style="top: auto; bottom: 0px; height: 3em;">
				<table style="height: 100%; margin:auto;"><tr><td>
				<base:icon image="annotation.png" style="vertical-align: text-bottom;"/> = annotation
				<base:icon image="parameter.png" style="vertical-align: text-bottom;"/> = protocol parameter<br>
				<base:icon image="item-error.png" style="vertical-align: text-bottom;" /> = not a parent item
				</td></tr></table>
			</div>
		
		</div>
		
		<div class="absolutefull" style="left: 28em; padding: 1em;">
			<div id="annotationType" style="font-weight: bold;"></div>
			<div id="annotationValues"></div>
		</div>
	
	</div>
	</form>
	</div>

	<base:buttongroup subclass="dialogbuttons">
		<base:button id="btnClone" title="Clone" image="copy.png" data-clone="1" />
		<base:button id="btnInherit" title="Inherit" image="inherit.png" />
		<base:button id="close" title="Cancel" />
	</base:buttongroup>
	
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>

