<%-- $Id: view_file.jsp 7760 2019-11-27 13:37:26Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.Application"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Group"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.File"
	import="net.sf.basedb.core.Location"
	import="net.sf.basedb.core.MultiPermissions"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.core.Project"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.ChangeHistoryUtil"
	import="net.sf.basedb.clients.web.PermissionUtil"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction" 
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.clients.web.extensions.fileviewer.FileViewerUtil"
	import="net.sf.basedb.clients.web.extensions.fileviewer.FileViewerContext"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.Collections"
	import="java.util.Date"
	import="java.util.Map"
	import="java.util.Set"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.FILE;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.ITEM);
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final float scale = Base.getScale(sc);
final String tab = Values.getString(request.getParameter("tab"), "properties");
final DbControl dc = sc.newDbControl();
try
{
	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);

	File file = File.getById(dc, itemId);
	String title = "File -- " + HTML.encodeTags(file.getName());
	String path = file.getPath().toString();
	String urlPath = HTML.urlEncode(path);
	String bytes = Values.formatBytes(file.getSize(), 2);
	
	final Location location = file.getLocation();
	final boolean writePermission = file.hasPermission(Permission.WRITE);
	final boolean deletePermission = file.hasPermission(Permission.DELETE);
	final boolean sharePermission = file.hasPermission(Permission.SET_PERMISSION);
	final boolean setOwnerPermission = file.hasPermission(Permission.SET_OWNER);
	final boolean isRemoved = file.isRemoved();
	final boolean isUsed = isRemoved && file.isUsed();
	final boolean deletePermanentlyPermission = deletePermission && !isUsed;
	final boolean isOwner = file.isOwner();
	final String mimeType = file.getMimeType();
	final boolean editable = writePermission && !file.isWriteProtected() &&
		mimeType != null &&  mimeType.startsWith("text/") && 
		file.getSize() < 102400 && file.getLocation() == Location.PRIMARY;

	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, guiContext, file);
	ExtensionsInvoker<ButtonAction> invoker = ToolbarUtil.useExtensions(jspContext);
	FileViewerContext fileContext = FileViewerContext.file();
	fileContext.setCurrentFile(file);
	FileViewerUtil fileViewer = new FileViewerUtil(jspContext, fileContext);
	%>
	<base:page type="iframe" title="<%=title%>" id="view-page">
	<base:head scripts="table.js,tabcontrol-2.js,dragdrop.js,~files.js" styles="table.css,toolbar.css,headertabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<div class="absolutefull auto-init" data-auto-init="drag-support">

		<div id="page-data" data-item-id="<%=itemId%>"></div>
		
		<t:tabcontrol 
			subclass="content mastertabcontrol"
			style="top: 0px;"
			id="main" active="<%=tab%>">
		<t:tab id="properties" title="Properties">
			<div>
			<table class="fullform bottomborder">
			<tr>
				<th class="itemstatus">
					<base:icon 
						image="shared.png" 
						visible="<%=file.isShared()%>"
						tooltip="This item is shared to other users, groups and/or projects"
					/>
					<base:icon 
						id="btnDeletePermanently"
						image="deleted.png"
						tooltip="This item has been flagged for deletion. Click to delete it now."
						enabled="<%=deletePermanentlyPermission %>"
						visible="<%=isRemoved%>" 
					/>
					<base:icon 
						id="btnUsingItems"
						image="used.png" 
						tooltip="This item is used by other items and can't be permanently deleted. Show the items that are using this one"
						visible="<%=isRemoved && isUsed%>" />
				</th>
				<td style="padding: 0px;">
					<tbl:toolbar subclass="bottomborder bg-filled-50">
						<tbl:button 
							id="btnEdit"
							disabled="<%=!writePermission%>" 
							image="edit.png" 
							title="Edit&hellip;" 
							tooltip="<%=writePermission ? "Edit this file" : "You do not have permission to edit this file"%>" 
						/>
						<tbl:button 
							id="btnEditFileData"
							visible="<%=editable%>"
							image="file_edit.png" 
							title="Edit file&hellip;" 
							tooltip="Edit the contents of this file" 
						/>
						<tbl:button 
							id="btnDelete"
							disabled="<%=!deletePermission%>" 
							image="delete.png" 
							title="Delete"
							visible="<%=!file.isRemoved()%>"
							tooltip="<%=deletePermission ? "Delete this file" : "You do not have permission to delete this file"%>" 
						/>
						<tbl:button 
							id="btnRestore"
							disabled="<%=!writePermission%>" 
							image="restore.png" 
							title="Restore"
							visible="<%=file.isRemoved()%>"
							tooltip="<%=writePermission ? "Restore this file" : "You do not have permission to restore this file"%>" 
						/>
						<tbl:button 
							id="btnShare"
							disabled="<%=!sharePermission%>"
							image="share.png"
							title="Share&hellip;" 
							tooltip="<%=sharePermission ? "Share this file to other user, groups and projects" : "You do not have permission to share this file"%>"
						/>
						<tbl:button 
							id="btnSetOwner"
							disabled="<%=!setOwnerPermission%>"
							image="take_ownership.png"
							title="Set owner&hellip;"
							tooltip="<%=setOwnerPermission ? "Change owner of this item" : "You do not have permission to change ownership of this item"%>"
						/>
						<tbl:button
							id="btnCompress"
							disabled="<%=!writePermission%>"
							image="compress.png"
							title="Compress&hellip;"
							visible="<%=!file.isCompressed() && location == Location.PRIMARY%>"
							tooltip="Store this file in a compressed format on the disk"
						/>
						<tbl:button
							id="btnDecompress"
							disabled="<%=!writePermission%>"
							image="decompress.png"
							title="Decompress&hellip;"
							visible="<%=file.isCompressed() && location == Location.PRIMARY%>"
							tooltip="Decompress this file and store it in it's normal format on the disk"
						/>
						<tbl:button 
							id="btnImport"
							image="import.png" 
							data-plugin-type="IMPORT"  
							title="Import&hellip;" 
							tooltip="Import data" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
						/>
						<tbl:button 
							id="btnExport"
							image="export.png"
							data-plugin-type="EXPORT" 
							title="Export&hellip;" 
							tooltip="Export data" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
						/>
						<tbl:button 
							id="btnRunPlugin"
							image="runplugin.png"  
							data-plugin-type="OTHER" 
							title="Run plugin&hellip;" 
							tooltip="Run a plugin" 
							visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
						/>
						<ext:render extensions="<%=invoker%>" context="<%=jspContext%>" 
							wrapper="<%=new PrefixSuffixRenderer<ButtonAction>(jspContext, "<td>", "</td>") %>"/>
						<tbl:button
							image="help.png"
							subclass="auto-init"
							data-auto-init="help"
							data-help-id="file.view.properties"
							title="Help&hellip;"
							tooltip="Get help about this page"
						/>
					</tbl:toolbar>
				</td>
			</tr>
			<tr>
				<th>File</th>
				<td><%=HTML.encodeTags(path)%></td>
			</tr>
			<tr>
				<th>Type</th>
				<td><base:propertyvalue item="<%=file%>" property="itemSubtype" /></td>
			</tr>
			<tr>
				<th>Write protected</th>
				<td><%=file.isWriteProtected() ? "yes" : "no"%></td>
			</tr>
			<tr>
				<th>Location</th>
				<td><%=location%>
				<%
				fileViewer.render();
				if (location == Location.PRIMARY && file.hasPermission(Permission.WRITE) && !file.isWriteProtected())
				{
					%> [<span class="link" 
						id="moveOffline"
						data-file-id="<%=itemId %>"
						data-location="OFFLINE"
						>move offline</span>
					| <span class="link auto-init"
						data-auto-init="upload-file"
						data-file-id="<%=itemId%>">replace</span>]
					<%
				}
				%>
				</td>
			</tr>
			<%
			if (location == Location.EXTERNAL)
			{
				%>
				<tr>
					<th class="subprompt">- URL</th>
					<td><%=HTML.niceFormat(file.getUrl(), HTML.LINK_URL)%></td>
				</tr>
				<tr>
					<th class="subprompt">- file server</th>
					<td><base:propertyvalue item="<%=file%>" property="fileServer" /></td>
				</tr>
				<%
			}
			%>
			<tr>
				<th>MIME type</th>
				<td><%=HTML.encodeTags(file.getMimeType())%></td>
			</tr>
			<tr>
				<th class="subprompt">- character set</th>
				<td><%=HTML.encodeTags(file.getCharacterSet() == null ? "n/a" : file.getCharacterSet())%></td>
			</tr>			
			<tr>
				<th>Size</th>
				<td title="<%=file.getSize()%> bytes"><%=bytes%></td>
			</tr>
			<%
			if (file.isCompressed())
			{
				%>
				<tr>
					<th class="subprompt">- compressed size</th>
					<td title="<%=file.getCompressedSize()%> bytes"><%=Values.formatBytes(file.getCompressedSize(), 2)%></td>
				</tr>
				<%
			}
			%>
			<tr>
				<th>Last update</th>
				<td><%=dateTimeFormatter.format(file.getLastUpdate())%></td>
			</tr>
			<tr>
				<th>MD5</th>
				<td><%=HTML.encodeTags(file.getMd5())%></td>
			</tr>
			<tr>
				<th>Owner</th>
				<td><base:propertyvalue item="<%=file%>" property="owner" /></td>
			</tr>
			<tr>
				<th>Permissions</th>
				<td><%=PermissionUtil.getFullPermissionNames(file)%></td>
			</tr>
			<tr>
				<th>Description</th>
				<td><%=HTML.niceFormat(file.getDescription())%></td>
			</tr>
			</table>
			</div>
			<jsp:include page="../../common/anytoany/list_anytoany.jsp">
				<jsp:param name="ID" value="<%=ID%>" />
				<jsp:param name="item_type" value="<%=itemType.name()%>" />
				<jsp:param name="item_id" value="<%=itemId%>" />
				<jsp:param name="title" value="Other items related to this file" />
			</jsp:include>
			<jsp:include page="../../common/share/list_share.jsp">
				<jsp:param name="ID" value="<%=ID%>" />
				<jsp:param name="item_type" value="<%=itemType.name()%>" />
				<jsp:param name="item_id" value="<%=itemId%>" />
				<jsp:param name="title" value="Shared to" />
			</jsp:include>
		</t:tab>
		<t:tab id="history" title="History"
			tooltip="Displays a log with the history of this item"
			visible="<%=ChangeHistoryUtil.showChangeHistoryTab(sc)%>">
			<jsp:include page="../../common/history/frameset.jsp">
				<jsp:param name="item_type" value="<%=itemType.name()%>" />
				<jsp:param name="item_id" value="<%=itemId%>" />
				<jsp:param name="ID" value="<%=ID%>" />
			</jsp:include>
		</t:tab>
		</t:tabcontrol>
		</div>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}

%>
