<%-- $Id: index.jsp 7596 2019-02-22 07:45:09Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Johan Enell

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------
	This is the welcome page once the user has logged in to BASE.
	It should do a lot more than display an empty page....

	...To be done....

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.Group"
	import="net.sf.basedb.core.Message"
	import="net.sf.basedb.core.Project"
	import="net.sf.basedb.core.News"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.Location"
	import="net.sf.basedb.core.Type"
	import="net.sf.basedb.core.Quota"
	import="net.sf.basedb.core.QuotaType"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Expression"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.servlet.RssNewsFeed"
	import="net.sf.basedb.util.Values"
	import="java.util.Date"	
	import="java.util.List"
	import="java.util.ArrayList"
	import="java.util.Set"
	import="java.util.HashSet"
%>
<%@ taglib
	prefix="base" uri="/WEB-INF/base.tld"
%>
<%!
private String getPrintableQuota(long myQuota, long groupQuota)
{
	String printable = "";
	if (myQuota == Quota.UNDEFINED && groupQuota == Quota.UNDEFINED)
	{
		// Both are undefined
		printable = "";
	}
	else if (myQuota < 0 && groupQuota < 0)
	{
		// both are unlimited or one is unlimited and the other is undefined
		printable = "unlimited";
	}
	else if (myQuota >= 0 && groupQuota >= 0)
	{
		// both are well-defined; use min value
		printable = Values.formatBytes(Math.min(myQuota, groupQuota));
	}
	else
	{
		// Only one is well-defined; use max value
		printable = Values.formatBytes(Math.max(myQuota, groupQuota));
	}
	return printable;
}
%>
<%
SessionControl sc = Base.getExistingSessionControl(pageContext, true);
String ID = sc.getId();

DbControl dc = null;
try
{
	dc = sc.newDbControl();
	User user = User.getById(dc, sc.getLoggedInUserId());
	Quota quota = user.getQuota();
	Group quotaGroup = user.getQuotaGroup();
	Quota groupQuota = quotaGroup == null ? null : quotaGroup.getQuota();
	
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);

	ItemQuery<Message> messageQuery = Message.getQuery(user);
	messageQuery.order(Orders.desc(Hql.property("timeSent")));
	messageQuery.restrict(Restrictions.eq(Hql.property("read"), Expressions.parameter("isread", false)));
	ItemResultList<Message> messages = messageQuery.list(dc);
	
	List<Project> projects = new ArrayList<Project>();
	String tmp = sc.getUserClientSetting("projects.recentActive");
	Set<Integer> recentProjects = new HashSet<Integer>();
	if (tmp != null)
	{
		for (String id : tmp.split(":"))
		{
			try
			{
				Project p = Project.getById(dc, Values.getInt(id));
				if (!p.isRemoved())
				{
					recentProjects.add(p.getId());
					projects.add(p);
				}
			}
			catch (RuntimeException ex)
			{}
		}
	}
	ItemQuery<Project> projectQuery = Project.getQuery();
	projectQuery.order(Orders.asc(Hql.property("name")));
	projectQuery.include(Include.MINE, Include.SHARED);
	projectQuery.restrict(Restrictions.not(Restrictions.in(Hql.property("id"), Expressions.parameter("projects"))));
	projectQuery.setParameter("projects", recentProjects, Type.INT);
	projects.addAll(projectQuery.list(dc));
	
	QuotaType totalQuotaType = QuotaType.getById(dc, SystemItems.getId(QuotaType.TOTAL));
	ItemQuery<QuotaType> quotaTypeQuery = QuotaType.getQuery();
	quotaTypeQuery.order(Orders.asc(Hql.property("id")));
	quotaTypeQuery.restrict(
		Restrictions.neq(
			Hql.property("id"), 
			Hql.entity(totalQuotaType)
		)
	);
	ItemResultList<QuotaType> quotaTypes = quotaTypeQuery.list(dc);

	ItemQuery<News> newsQuery = News.getQuery();
	Expression today = Expressions.parameter("today");
	newsQuery.restrict(Restrictions.lteq(Hql.property("startDate"), today));
	newsQuery.restrict(
		Restrictions.or(
			Restrictions.gteq(Hql.property("endDate"), today),
			Restrictions.eq(Hql.property("endDate"), null)
		)
	);
	newsQuery.order(Orders.desc(Hql.property("sticky")));
	newsQuery.order(Orders.desc(Hql.property("newsDate")));
	newsQuery.order(Orders.desc(Hql.property("id")));
	newsQuery.setParameter("today", new Date(), Type.DATE);
	ItemResultList<News> news = newsQuery.list(dc);
	
	String help = Values.getStringOrNull(sc.getClientDefaultSetting("server.links.help"));
	String faq = Values.getStringOrNull(sc.getClientDefaultSetting("server.links.faq"));
	String reportBug = Values.getStringOrNull(sc.getClientDefaultSetting("server.links.reportbug"));
	boolean hasHelp = help != null || faq != null || reportBug != null;
%>
<base:page title="Welcome to BASE" id="homepage">
<base:head styles="login.css" scripts="~home.js"/>
<base:body>

	<h1>Welcome to BASE</h1>

	<div class="content">
	
	<div class="absolutefull" style="width: 50%;">
	
		<div class="absolutefull" style="left: 1em; right: 0.5em; bottom: <%=hasHelp ? "16em" : "1em" %>;">

		<div class="absolutefull" style="height: 50%;">
			<div class="absolutefull" style="height: 2em; overflow: hidden;">
				<h3 style="height: 100%;"><base:icon image="project.png" />Projects (<%=projects.size()%>)</h3>
			</div>
			<div id="projectsSection" class="absolutefull welcomesection" style="top: 2em; bottom: 0.5em;">
			<%
			if (projects.size() == 0)
			{
				%>
				You are not member of any projects.
				<%
			}
			else
			{
				int activeProjectId = sc.getActiveProjectId();
				%>
				<table style="border-spacing: 2px;">
				<%
				if (activeProjectId == 0)
				{
					%>
					<td><base:icon image="warning.png" /></td>
					<td><i>No active project</i></td>
					<td>&nbsp;</td>
					<%
				}
				for (Project p : projects)
				{
					int projectId = p.getId();
					if (projectId == activeProjectId)
					{
						%>
						<tr>
							<td><base:icon image="bullet.png" /></td>
							<td><b><%=Base.getLinkedName(ID, p, false, true)%></b></td>
							<td>[active]</td>
						</tr>
						<%
					}
					else
					{
						%>
						<tr>
							<td>&nbsp;</td>
							<td><%=Base.getLinkedName(ID, p, false, true)%></td>
							<td>[<span class="link auto-init" data-auto-init="activate-project" 
								data-project-id="<%=projectId%>">set active</span>]</td>
						</tr>
						<%
					}
				}
				%>
				</table>
				<%
			}
			%>
			</div>
		</div>

		<div class="absolutefull" style="height: 50%; top: auto;">
			<div class="absolutefull" style="height: 2em; top: 0.5em; overflow: hidden;">
				<h3 style="height: 100%;"><base:icon image="message.png" />New messages (<%=messages.size()%>)</h3>
			</div>
			<div id="messagesSection" class="absolutefull news welcomesection" style="top: 2.5em;">
			<%
			String broadcastTitle = (String)application.getAttribute("broadcast.title");
			if (broadcastTitle != null)
			{
				String broadcastMessage = (String)application.getAttribute("broadcast.message");
				%>
				<div class="item note sticky">
					<div class="headline">
						<span class="date"><%=dateFormatter.format(new Date())%></span>
						<%=HTML.encodeTags(broadcastTitle)%>
					</div>
					<div class="text"><%=HTML.niceFormat(broadcastMessage)%></div>
				</div>
				<% 
			}
			if (messages.size() == 0 && broadcastTitle == null)
			{
				%>
				No new messages.
				<%
			}
			else
			{
				for (Message m : messages)
				{
					String fullName = m.getName();
					String text = Values.trimString(m.getDescription(), 100);
					%>
					<div class="item">
						<div class="headline">
							<span class="link auto-init" data-auto-init="item-link" 
								data-item-type="MESSAGE" data-item-id="<%=m.getId()%>" 
								title="<%=HTML.encodeTags(fullName)%>">
							<span class="date"><%=dateTimeFormatter.format(m.getTimeSent())%></span>
							<%=HTML.encodeTags(fullName)%></a>
						</div>
						<div class="text"><%=HTML.encodeTags(text)%></div>
					</div>
					<%
				}
			}
			%>
			</div>
		</div>
	</div>
	
	<%
	if (hasHelp)
	{
		%>
		<div class="absolutefull" style="left: 1em; right: 0.5em; top: auto; height: 14em; bottom: 1em; ">
			<div class="absolutefull" style="height: 2em; overflow: hidden;">
			<h3 style="height: 100%;"><base:icon image="help.png" />Help</h3>
			</div>
			<div id="helpSection" class="absolutefull welcomesection" style="top: 2em;">
			<%
			if (help != null)
			{
				%>
				<a href="<%=help%>" target="Help"><base:icon image="bullet.png" />&nbsp;Help&hellip;</a><br>
				<%
			}
			%>
			<%
			if (faq != null)
			{
				%>
				<a href="<%=faq%>" target="FAQ"><base:icon image="bullet.png" />&nbsp;Frequently asked questions&hellip;</a><br>
				<%
			}
			%>
			<%
			if (reportBug != null)
			{
				%>
				<a href="<%=reportBug%>" target="ReportBug"><base:icon image="bug.png" />&nbsp;Report a bug&hellip;</a><br>
				<%
			}
			%>
			</div>
		</div>
		<%
	}
	%>
	
	</div>
	<div class="absolutefull" style="width: 50%; left: auto;">
	
		<div class="absolutefull" style="left: 0.5em; right: 1em; bottom: 16em;">
			<div class="absolutefull" style="height: 2em; overflow: hidden;">
			<h3 style="height: 100%;"><base:icon image="news.png" />News and announcements (<%=news.size()%>) 
				<%
				if (RssNewsFeed.isEnabled()) 
				{
					%>
					<a href="../info/news.rss" 
						title="Subscribe to news from this BASE server"><base:icon image="rss.png" style="float: right;"/></a>
					<%
				}
				%>
			</h3>
			</div>
			<div id="newsSection" class="absolutefull news welcomesection" style="top: 2em;">
				<%
				if (news.size() == 0)
				{
					%>
					Nothing new.
					<%
				}
				else
				{
					for (News n : news)
					{
						%>
						<div class="item <%=n.isSticky() ? "note sticky" :""%>">
							<div class="headline">
								<span class="date"><%=dateFormatter.format(n.getNewsDate())%></span>
								<%=HTML.encodeTags(n.getName())%>
							</div>
							<div class="text"><%=HTML.niceFormat(n.getDescription())%></div>
						</div>
						<%
					}
				}
				%>
			</div>
		</div>
	
		<div class="absolutefull" style="left: 0.5em; right: 1em; top: auto; height: 14em; bottom: 1em;">
			<div class="absolutefull" style="height: 2em; overflow: hidden;">
			<h3 style="height: 100%;"><base:icon image="diskusage.png" />Disk usage</h3>
			</div>
			<div id="quotaSection" class="absolutefull welcomesection" style="top: 2em;">
			<%
			if (quotaGroup != null)
			{
				%>
				Using quota from group: <%=Base.getLinkedName(ID, quotaGroup, false, true)%>
				<%
			}
			%>
			<table style="width: 100%; border-bottom-width: 1px; border-bottom-style: dotted; margin-bottom: 0.5em;">
			<tr style="text-align: center;">
				<th></th>
				<th colspan="2">Primary location</th>
				<th>External</th>
			</tr>
			<tr style="text-align: center;">
				<th></th>
				<th>Used</th>
				<th>Assigned</th>
				<th>Used</th>
			</tr>
			<%
				long myPrimaryUsage = user.getDiskUsage(totalQuotaType, Location.PRIMARY);
				long myPrimaryQuota = quota.getQuotaValue(totalQuotaType, Location.PRIMARY);
				long groupPrimaryQuota = groupQuota == null ? Quota.UNDEFINED : groupQuota.getQuotaValue(totalQuotaType, Location.PRIMARY);
				long myExternalUsage = user.getDiskUsage(totalQuotaType, Location.EXTERNAL);
			%>
			<tr style="text-align: center; border-bottom-width: 1px; border-bottom-style: dotted;">
				<th>Total</th>
				<td><%=getPrintableQuota(myPrimaryUsage, Quota.UNDEFINED)%></td>
				<td><%=getPrintableQuota(myPrimaryQuota, groupPrimaryQuota)%></td>
				<td><%=getPrintableQuota(myExternalUsage, Quota.UNDEFINED)%></td>
			</tr>
			<%
			for (QuotaType qt : quotaTypes)
			{
				myPrimaryUsage = user.getDiskUsage(qt, Location.PRIMARY);
				myPrimaryQuota = quota.getQuotaValue(qt, Location.PRIMARY);
				groupPrimaryQuota = groupQuota == null ? Quota.UNDEFINED : groupQuota.getQuotaValue(qt, Location.PRIMARY);
				myExternalUsage = user.getDiskUsage(qt, Location.EXTERNAL);
				%>
				<tr style="text-align: center;">
					<th><%=HTML.encodeTags(qt.getName())%></th>
					<td><%=getPrintableQuota(myPrimaryUsage, Quota.UNDEFINED)%></td>
					<td><%=getPrintableQuota(myPrimaryQuota, groupPrimaryQuota)%></td>
					<td><%=getPrintableQuota(myExternalUsage, Quota.UNDEFINED)%></td>
				</tr>
				<%
			}
			%>
			</table>
			<base:buttongroup>
				<base:button id="btnViewDiskUsage" image="bullet.png" title="View details" />
			</base:buttongroup>
			</div>
		</div>
	</div>
	</div>
</base:body>
</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
