<%-- $Id: settings.jsp 7605 2019-02-26 10:10:15Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	This page is used to display and modify the attributes of
	the logged in user. It displays a tabbed dialogue:

	@param page
		The name active page of the tabbed dialog.
		- contact: Contact information for the user, address, phone, email, etc.
		- password: The password for the user
		- other: Other information

	Saving the form invokes the submit_user.jsp page with cmd=SaveSettings

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.*"
	import="net.sf.basedb.util.EmailUtil"
	import="net.sf.basedb.clients.web.*"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.formatter.Formatter"	
	import="net.sf.basedb.util.json.JsonUtil"
	import="net.sf.basedb.util.json.JsonConverter"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.formatter.FormatterSettings"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.tabcontrol.TabAction"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="org.json.simple.JSONObject"
	import="org.json.simple.JSONArray"
	import="java.util.Date"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String activePage = request.getParameter("page");
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
final ItemContext cc = Base.getAndSetCurrentContext(sc, Item.USER, null, null);
try
{
	final User user = User.getById(dc, sc.getLoggedInUserId());
	final String title = "Information for " + HTML.encodeTags(user.getName());
	final boolean writePermission = user.hasPermission(Permission.RESTRICTED_WRITE);
	final String ifDisabled = writePermission ? "" : "disabled";
	final String passwordDisabled = writePermission ? "" : "disabled";
	dc.detachItem(user);
	sc.setSessionSetting("user", user);
	
	List<ExtendedProperty> extendedProperties = ExtendedProperties.getProperties("UserData");
	JSONArray jsonExtendedProperties = JsonUtil.toArray(extendedProperties, new JsonConverter<ExtendedProperty>()
		{
			public Object convert(ExtendedProperty ep)
			{
				if (ep.isRestrictedEdit()) return null;
				JSONObject json = new JSONObject();
				json.put("name", ep.getName());
				json.put("valueType", ep.getType().name());
				json.put("title", ep.getTitle());
				json.put("nullable", ep.isNullable());
				return json;
			}
		});
	
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	String dateFormat = FormatterSettings.getDateFormat(sc);
	String jsDateFormat = HTML.javaScriptEncode(dateFormat);
	String htmlDateFormat = HTML.encodeTags(dateFormat);
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	String dateTimeFormat = FormatterSettings.getDateTimeFormat(sc);
	String jsDateTimeFormat = HTML.javaScriptEncode(dateTimeFormat);
	String htmlDateTimeFormat = HTML.encodeTags(dateTimeFormat);
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, null, user);
	ExtensionsInvoker<TabAction> tabsInvoker = ExtensionsControl.useExtensions(jspContext, EditUtil.EP_PREFIX_EDIT+"user-information");
%>
	<base:page type="popup" title="<%=title%>">
	<base:head scripts="tabcontrol-2.js,~settings.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%><base:help tabcontrol="settings" /></h1>
	
		<div id="page-data" class="datacontainer"
			data-date-format="<%=htmlDateFormat%>"
			data-datetime-format="<%=htmlDateTimeFormat%>"
			data-extended-properties="<%=HTML.encodeTags(jsonExtendedProperties.toJSONString())%>"
		></div>
	
		<form action="submit_user.jsp?ID=<%=ID%>" method="post" name="user">
		<input type=hidden name="cmd" value="SaveSettings">
	
		<t:tabcontrol 
			subclass="content dialogtabcontrol"
			active="<%=activePage%>" id="settings" 
			extensions="<%=tabsInvoker%>"
			position="bottom" remember="false">
	
			<t:tab id="contact" title="Contact information" helpid="userpreferences.contact">
				<table class="fullform input100">
				<tr>
					<th>Full name</th>
					<td><%=HTML.encodeTags(user.getName())%></td>
					<td></td>
				</tr>
				<tr>
					<th>Email</th>
					<td><input <%=ifDisabled%> type="text" name="email" value="<%=HTML.encodeTags(user.getEmail())%>" maxlength="<%=User.MAX_EMAIL_LENGTH%>"></td>
					<td></td>
				</tr>
				<%
				if (EmailUtil.isEnabled() && writePermission)
				{
					%>
					<tr>
						<th class="subprompt"></th>
						<td>
							<%
							boolean sendMessagesAsEmail = user.getSendMessagesAsEmail();
							%>
							<input type="checkbox" name="sendMessagesAsEmail" id="sendMessagesAsEmail" value="1" <%=sendMessagesAsEmail ? "checked" : ""%>
								><label for="sendMessagesAsEmail">Send system messages as email</label>
						</td>
						<td></td>
					</tr>
					<tr>
						<th class="subprompt"></th>
						<td>
							<%
							boolean useDeviceVerification = user.getUseDeviceVerification();
							%>
							<input type="checkbox" name="useDeviceVerification" id="useDeviceVerification" value="1" <%=useDeviceVerification ? "checked" : ""%>
								><label for="useDeviceVerification">Enable 2-factor login</label>
						</td>
						<td></td>
					</tr>
					<%
				}
				%>
				<tr>
					<th>Organisation</th>
					<td><input <%=ifDisabled%> type="text" name="organisation" value="<%=HTML.encodeTags(user.getOrganisation())%>" maxlength="<%=User.MAX_ORGANISATION_LENGTH%>"></td>
					<td></td>
				</tr>
				<tr class="big">
					<th>Address</th>
					<td>
						<textarea <%=ifDisabled%> rows="4" name="address" id="address"><%=HTML.encodeTags(user.getAddress())%></textarea>
					</td>
					<td style="width: 20px;">
						<base:zoom textarea="address" title="Address" />
					</td>
				</tr>
				<tr>
					<th>Phone</th>
					<td><input <%=ifDisabled%> type="text" name="phone" value="<%=HTML.encodeTags(user.getPhone())%>" maxlength="<%=User.MAX_PHONE_LENGTH%>"></td>
					<td></td>
				</tr>
				<tr>
					<th>Fax</th>
					<td><input <%=ifDisabled%> type="text" name="fax" value="<%=HTML.encodeTags(user.getFax())%>" maxlength="<%=User.MAX_FAX_LENGTH%>"></td>
					<td></td>
				</tr>
				<tr>
					<th>Url</th>
					<td><input <%=ifDisabled%> type="text" name="url" value="<%=HTML.encodeTags(user.getUrl())%>" maxlength="<%=User.MAX_URL_LENGTH%>"></td>
					<td></td>
				</tr>
				<tr class="dynamic">
					<th></th>
					<td></td>
					<td></td>
				</tr>
				</table>
			</t:tab>
			
			<t:tab id="password" title="Password" helpid="userpreferences.password">
				<table class="fullform input100">
				<tr>
					<th>Login</th>
					<td><%=HTML.encodeTags(user.getLogin())%></td>
				</tr>
				<tr>
					<th>New password</th>
					<td><input <%=passwordDisabled%> type="password" name="password" value=""  maxlength="30"></td>
				</tr>
				<tr>
					<th>Retype password</th>
					<td><input <%=passwordDisabled%> type="password" name="retype_password" value="" maxlength="30"></td>
				</tr>
				<tr class="dynamic">
					<th></th>
					<td><i>- leave empty to not change the password</i></td>
				</tr>
				</table>
			</t:tab>
		
			<t:tab id="other" title="Other information" 
				helpid="userpreferences.other">
				<table class="fullform input100">
				<%
				if (extendedProperties != null)
				{
					for (ExtendedProperty ep : extendedProperties)
					{
						String name = ep.getName();
						String fieldName = "ep."+name;
						Type type = ep.getType();
						boolean required = !ep.isNullable();
						Object value = user == null ? cc.getPropertyValue(name) : user.getExtended(name);
						String isRequired = required ? "class=\"required\"" : "";
						final String isDisabled = ep.isRestrictedEdit() ? "disabled" : "";
						boolean addZoom = false;
						%>
						<tr>
							<th><%=HTML.encodeTags(ep.getTitle())%></th>
							<td>
							<%
							if (type == Type.INT || type == Type.LONG)
							{
								%>
								<input <%=isRequired%> <%=isDisabled %> type="text" name="<%=fieldName%>" id="<%=fieldName%>"
									value="<%=value == null ? "" : value%>" style="width: 15em;" maxlength="20">
								<%
							}
							else if (type == Type.FLOAT || type == Type.DOUBLE)
							{
								%>
								<input <%=isRequired%> <%=isDisabled%> type="text" name="<%=fieldName%>" id="<%=fieldName%>"
									value="<%=value == null ? "" : value%>" style="width: 15em;" maxlength="20">
								<%
							}
							else if (type == Type.STRING)
							{
								%>
								<input <%=isRequired%> <%=isDisabled%> type="text" name="<%=fieldName%>"
									value="<%=HTML.encodeTags((String)value)%>"
									maxlength="<%=ep.getLength()%>">
								<%
							}
							else if (type == Type.TEXT)
							{
								addZoom = true;
								%>
								<textarea <%=isRequired%> <%=isDisabled%> name="<%=fieldName%>" id="<%=fieldName%>" rows="6"  
									><%=HTML.encodeTags((String)value)%></textarea>
								<%
							}
							else if (type == Type.BOOLEAN)
							{
								Boolean b = (Boolean)value;
								if (!required)
								{
									%>
									<input type="radio" <%=isDisabled%> name="<%=fieldName%>" id="<%=fieldName%>.null" value=""
										<%=b == null ? "checked" : ""%> 
										><label for="<%=fieldName%>.null"><i>- not specified -</i></label><br>
									<%
								}
								%>
								<input type="radio" <%=isDisabled%> name="<%=fieldName%>" id="<%=fieldName%>.true" value="true" 
									<%=b != null && b == true ? "checked" : ""%>
									><label for="<%=fieldName%>.true">true</label>
								<input type="radio" <%=isDisabled%> name="<%=fieldName%>" id="<%=fieldName%>.false" value="false" 
									<%=b != null && b == false || required && b == null ? "checked" : ""%>
									><label for="<%=fieldName%>.false">false</label>
								<%
							}
							else if (type == Type.DATE)
							{
								%>
								<table>
								<tr>
								<td>
									<input <%=isRequired%> <%=isDisabled%> type="text" name="<%=fieldName%>" id="<%=fieldName%>"
										value="<%=dateFormatter.format((Date)value)%>" style="width: 15em;"
										maxlength="20" title="Enter date in format: <%=dateFormat%>"
										>
								</td>
								<td>
									<base:button 
										id="<%="btn."+fieldName%>"
										data-extended-property="<%=name%>"
										image="calendar.png"
										title="Calendar&hellip;" 
										tooltip="Select a date from a calendar" 
										visible="<%=writePermission%>"
									/>
								</td>
								</tr>
								</table>
								<%
							}
							else if (type == Type.TIMESTAMP)
							{
								%>
								<table>
								<tr>
								<td>
									<input <%=isRequired%> <%=isDisabled%> type="text" name="<%=fieldName%>" id="<%=fieldName%>"
										value="<%=dateTimeFormatter.format((Date)value)%>" style="width: 15em;"
										maxlength="20" title="Enter timestamp in format: <%=dateTimeFormat%>"
										>
								</td>
								<td>
									<base:button 
										id="<%="btn."+fieldName%>"
										data-extended-property="<%=name%>"
										image="calendar.png"
										title="Calendar&hellip;" 
										tooltip="Select a timestamp from a calendar" 
										visible="<%=writePermission%>"
									/>
								</td>
								</tr>
								</table>
								<%
							}
							%>
							</td>
							<td style="width: 20px;">
								<%
								if (addZoom)
								{
									%>
									<base:zoom textarea="<%=fieldName%>" title="<%=HTML.encodeTags(ep.getTitle()) %>" />
									<%
								}
								%>
							</td>
						</tr>
						<%
					}
				}
				%>
				<tr class="dynamic">
					<th>Description</th>
					<td>
						<textarea <%=ifDisabled%> rows="5" name="description" id="description"><%=HTML.encodeTags(user.getDescription())%></textarea>
					</td>
					<td style="width: 20px;">
						<base:zoom textarea="description" title="Description" />
					</td>
				</tr>
				</table>
			</t:tab>
		</t:tabcontrol>
		</form>
	
		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" visible="<%=writePermission%>"/>
			<base:button id="close" title="Cancel" visible="<%=writePermission%>"/>
			<base:button id="close" title="Close" visible="<%=!writePermission%>"/>
		</base:buttongroup>
	
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
