/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var View = function()
{
	var view = {};
	
	/**
		Initialize the page.
	*/
	view.initPage = function()
	{
		
		TabControl.addTabActivateListener('explorer.search', view.viewSearch);
	
		var frm = document.forms['reporter'];
		frm.cmd.value = 'SetReporterColumns';
		Buttons.addClickHandler('btnReporterColumns', Buttons.configureColumns, {'table-id': 'reporter', 'settings': 'reporter-columns'});
		Buttons.addClickHandler('btnSpotColumns', Buttons.configureColumns, {'table-id': 'spotdata'});
		
		Events.addEventHandler('newReporterIndex', 'keypress', Events.integerOnly);
		Events.doOnEnter('newReporterIndex', view.gotoReporterIndex);

		Events.addEventHandler('newSpotIndex', 'keypress', Events.integerOnly);
		Events.doOnEnter('newSpotIndex', view.gotoSpotIndex);

		Buttons.addClickHandler('btnAssayPlot', view.assayPlot);

		view.initAnnotationSummary();
	}

	view.initElements = function(element, autoInit)
	{
		if (autoInit == 'annotation-plot')
		{
			Buttons.addClickHandler(element, view.annotationPlot);
		}
	}
	
	view.gotoReporterIndex = function(event)
	{
		var frm = document.forms['reporterIndex'];
		var reporterIndex = parseInt(frm.newReporterIndex.value);
		var maxIndex = Data.int(event.currentTarget, 'max-index');
		if (reporterIndex < 1 || reporterIndex > maxIndex)
		{
			Forms.showNotification(frm.newReporterIndex, 'The reporter index must be between 1 and ' + maxIndex);
			return false;
		}
		frm.cmd.value = 'SetReporterIndex';
		Forms.addHidden(frm, 'reporterIndex', reporterIndex-1);
		frm.submit();
	}

	view.gotoSpotIndex = function(event)
	{
		var frm = document.forms['spotIndex'];
		var spotIndex = parseInt(frm.newSpotIndex.value);
		var maxIndex =  Data.int(event.currentTarget, 'max-index');
		if (spotIndex < 1 || spotIndex > maxIndex)
		{
			Forms.showNotification(frm.newSpotIndex, 'The spot index must be between 1 and ' + maxIndex);
			return false;
		}
		frm.cmd.value = 'SetSpotIndex';
		Forms.addHidden(frm, 'spotIndex', spotIndex-1);
		frm.submit();
	}

	view.viewSearch = function()
	{
		var url = '../search/index.jsp?ID='+App.getSessionId();
		url += '&bioassayset_id='+Data.get('page-data', 'bioassayset-id');
		location.href = url;
	}

	view.assayPlot = function()
	{
		var frm = document.forms['spotdata'];
		
		var url = 'plotter.jsp?ID='+App.getSessionId();
		url += '&type=assay';
		url += '&bioAssaySetId='+Data.get('page-data', 'bioassayset-id');
		url += '&reporterIndex='+frm.reporter_index.value;
		url += '&positionIndex='+frm.position_index.value;
		Dialogs.openPopup(url, 'AssayPlot', 1250, 700);
	}
	
	view.annotationPlot = function(event)
	{
		var frm = document.forms['spotdata'];
		var url = 'plotter.jsp?ID='+App.getSessionId();
		url += '&type=annotation';
		url += '&bioAssaySetId='+Data.get('page-data', 'bioassayset-id');
		url += '&reporterIndex='+frm.reporter_index.value;
		url += '&positionIndex='+frm.position_index.value;
		url += '&annotationTypeId=' + Data.get(event.currentTarget, 'annotation-type');
		Dialogs.openPopup(url, 'AnnotationPlot', 1250, 700);
	}
	
	view.initAnnotationSummary = function()
	{
		var summary = Data.json('annotation-summary', 'summary');
		var channels = Data.int('annotation-summary', 'channels');
		
		for (var i = 0; i < summary.length; i++)
		{
			var at = summary[i];
			
			var groups = at.groups;
			for (var g = 0; g < groups.length; g++)
			{
				var group = groups[g];
				var elementId = at.id+'.'+group.id;
				Doc.element('count.'+elementId).innerHTML = group.count;
				
				for (var ch = 1; ch <= channels; ch++)
				{
					Doc.element('mean.'+elementId+'.ch'+ch).innerHTML = group.mean[ch-1];
				}
				
				if (group.gmean)
				{
					Doc.element('gmean.'+elementId).innerHTML = group.gmean;
					Doc.element('sd.'+elementId).innerHTML = group.sd;
				}
			}
		}
	}
	
	return view;
}();

Doc.onLoad(View.initPage);
Doc.addElementInitializer(View.initElements);
